import logging as log


def check_xnat_item(x, a):
    experiments = x.array.experiments()
    projects = [e['project'] for e in experiments]
    experiments = [e['ID'] for e in x.array.experiments()]

    result = -1
    if a in projects:
        log.info('Project detected: %s' % a)
        result = 0
    elif a in experiments:
        log.info('Experiment detected: %s' % a)
        result = 1
    else:
        from bx import lists
        if hasattr(lists, a):
            print('List detected: %s' % a)
            result = 2

    return result


def collect_experiments(x, id, columns=['label', 'subject_label'], max_rows=1):
    import os
    t = check_xnat_item(x, id)

    if not os.environ.get('CI_TEST', None):
        max_rows = None

    experiments = []
    if t == 0:
        experiments = x.array.experiments(project_id=id,
                                          columns=columns).data[:max_rows]
    elif t == 1:
        experiments = [x.array.experiments(experiment_id=id,
                                           columns=columns).data[0]]
    elif t == 2:
        from bx import lists
        expes = getattr(lists, id)
        for e in expes[:max_rows]:
            ex = x.array.experiments(experiment_id=e,
                                     columns=columns).data[0]
            experiments.append(ex)
    else:
        msg = '%s is not a project or an experiment nor a list' % id
        raise ValueError(msg)

    return experiments



from bx.command import Command
from bx import download as dl
import os
import logging as log


class QSMxTCommand(Command):
    """QSMxT - QSM processing and analysis pipeline. Quantitative susceptibility
    mapping (QSM) is an MRI technique for quantifying magnetic susceptibility
    within biological tissue.

    Available subcommands:
     stats:\t\tcreates an Excel table with regional QSM values (in FreeSurfer aseg atlas).
     maps:\t\tdownload the reconstructed QSM maps
     files:\t\tdownload all QSMxT outputs (QSM maps, segmentations, everything...)
     report:\t\tdownload the validation report issued by `QSMxTValidator`
     tests:\t\tcreate an Excel table with all automatic tests outcomes from bbrc-validator

    Usage:
     bx qsmxt <subcommand> <resource_id>

    References:
    - Stewart AW et al., Magnetic Resonance in Medicine, 2022.
    """
    nargs = 2
    resource_name = 'QSMXT'
    subcommands = ['stats', 'maps', 'files', 'report', 'tests']
    validator = 'QSMxTValidator'
    url = 'https://gitlab.com/bbrc/xnat/docker-images/-/tree/master/qsmxt'

    def __init__(self, *args, **kwargs):
        super(QSMxTCommand, self).__init__(*args, **kwargs)

    def parse(self):
        subcommand = self.args[0]
        id = self.args[1]

        if subcommand == 'stats':
            sf = 'qsmxt_%s' % subcommand
            df = self.run_id(id, dl.measurements, subfunc=sf,
                             resource_name='QSMXT', max_rows=10)
            self.to_excel(df)

        elif subcommand == 'maps':
            debug = bool(os.environ.get('CI_TEST', None))
            self.run_id(id, download_qsm, resource_name=self.resource_name,
                        destdir=self.destdir, subcommand=subcommand)

        elif subcommand in ['files', 'report']:
            self.run_id(id, dl.download, resource_name=self.resource_name,
                        validator=self.validator, destdir=self.destdir,
                        subcommand=subcommand)

        elif subcommand == 'tests':

            version = ['8ea92b93', '*']
            from bx import validation as val
            df = self.run_id(id, val.validation_scores,
                             validator=self.validator,
                             version=version, max_rows=25)
            self.to_excel(df)


def download_qsm(x, experiments, resource_name, destdir, subcommand):

    import os.path as op
    from tqdm import tqdm
    from pyxnat.core.errors import DataError

    filename = 'qsm/*_part-phase_MEGRE_scaled_qsm-filled_000_average.nii.gz'

    for e in tqdm(experiments):
        log.debug('Experiment %s:' % e['ID'])

        r = x.select.experiment(e['ID']).resource(resource_name)
        files = r.files(filename)

        for f in files:
            fn = op.basename(f.label())
            try:
                f.get(op.join(destdir, fn))
            except DataError as exc:
                log.error('Failed for %s. Skipping it. (%s)' % (e, exc))



from bx.command import Command
from bx import download as dl


class CAT12Command(Command):
    """CAT12 - Gray/white matter segmentation

    Available subcommands:
     files:\t\tdownload all `CAT12` outputs (segmentation maps, warp fields, everything...)
     volumes:\t\tcreates an Excel table with GM/WM/CSF volumes
     snapshot:\t\tdownload a snapshot from the segmentation results
     report:\t\tdownload the validation report issued by `CAT12Validator`
     tests:\t\tcreates an Excel table with all automatic tests outcomes from `CAT12Validator`

    Usage:
     bx cat12 <subcommand> <resource_id>
    """
    nargs = 2
    resource_name = 'CAT12_SEGMENT'
    subcommands = ['volumes', 'files', 'report', 'snapshot', 'tests', 'rc']
    validator = 'CAT12SegmentValidator'
    url = 'https://gitlab.com/bbrc/xnat/xnat-pipelines/-/tree/master/cat'

    def __init__(self, *args, **kwargs):
        super(CAT12Command, self).__init__(*args, **kwargs)

    def parse(self):
        subcommand = self.args[0]
        id = self.args[1]

        if subcommand in ['files', 'report', 'snapshot']:
            self.run_id(id, dl.download, resource_name=self.resource_name,
                        validator=self.validator, destdir=self.destdir,
                        subcommand=subcommand)

        elif subcommand in ['rc']:
            self.run_id(id, dl.download, resource_name=self.resource_name,
                        validator=self.validator, destdir=self.destdir,
                        subcommand='rc')

        elif subcommand == 'volumes':
            df = self.run_id(id, dl.measurements, subfunc=subcommand,
                             resource_name=self.resource_name)
            self.to_excel(df)

        elif subcommand == 'tests':
            version = ['*']
            from bx import validation as val
            df = self.run_id(id, val.validation_scores,
                             validator=self.validator,
                             version=version, max_rows=25)
            self.to_excel(df)



from bx.command import Command


class ArchivingCommand(Command):
    """Archiving - used to collect automatic tests from `ArchivingValidator`

    Available subcommands:
     tests:\t\tcreates an Excel table with all automatic tests outcomes from `ArchivingValidator`

    Usage:
     bx archiving <subcommand> <resource_id>
    """
    nargs = 2
    validator_name = 'ArchivingValidator'
    subcommands = ['tests']

    def __init__(self, *args, **kwargs):
        super(ArchivingCommand, self).__init__(*args, **kwargs)

    def parse(self):
        subcommand = self.args[0]
        id = self.args[1]
        if subcommand == 'tests':
            version = ['*', '0390c55f']

            from bx import validation as val
            df = self.run_id(id, val.validation_scores,
                             validator=self.validator_name,
                             version=version, max_rows=25)
            self.to_excel(df)



from bx.command import Command
from tqdm import tqdm
import pandas as pd
from bx import download as dl
import bx
import logging as log


braak_I_II_ctx = ['entorhinal']
braak_III_IV_ctx = ['fusiform', 'lingual', 'parahippocampal', 'temporalpole',
                    'caudalanteriorcingulate', 'inferiortemporal', 'insula',
                    'isthmuscingulate', 'middletemporal', 'posteriorcingulate',
                    'rostralanteriorcingulate']
braak_V_VI_ctx = ['bankssts', 'caudalmiddlefrontal', 'cuneus',
                  'inferiorparietal', 'lateraloccipital',
                  'lateralorbitofrontal', 'medialorbitofrontal', 'paracentral',
                  'pericalcarine', 'postcentral', 'precentral', 'precuneus',
                  'rostralmiddlefrontal', 'superiorfrontal',
                  'superiorparietal', 'superiortemporal', 'frontalpole',
                  'transversetemporal']

braak_I_II = ['Right-Hippocampus', 'Left-Hippocampus']
braak_III_IV = ['Left-Amygdala', 'Right-Amygdala', 'Left-Thalamus-Proper',
                'Right-Thalamus-Proper']
braak_V_VI = ['Right-Caudate', 'Left-Caudate', 'Right-Putamen',
              'Left-Putamen', 'Right-Accumbens-area',
              'Left-Accumbens-area']


class BraakCommand(Command):
    """Extract morphometric/metabolic measurements based on Braak staging.

    Morphometric values are based on regional volumes or cortical thickness
    as estimated individually by FreeSurfer with respect to each specific stage
     (namely, Braak_I_II, Braak_III_IV and Braak_V_VI). For each of them, the
    mean value in all the regions related to the specific stage is returned.

    Metabolic data refer to FDG update associated with each stage as defined
    by their corresponding ROIs. Masks of each stage were defined based on the
    CerebrA atlas (Manera et al.).

    Available subcommands:
     volumes:\t\tcreates an Excel table with regional volumes for each stage
     thickness:\t\tcreates an Excel table with cortical thickness for each stage
     fdg:\t\tcreates an Excel table with the FDG uptake for each stage

    Usage:
     bx braak <subcommand> <resource_id>

    References:
    - Braak et al., Acta Neuropathol. 2006
    - Schöll et al., Neuron. 2016
    - Manera et al.,  Scientific Data. 2020
    """
    nargs = 2
    subcommands = ['volumes', 'thickness', 'fdg']

    def __init__(self, *args, **kwargs):
        super(BraakCommand, self).__init__(*args, **kwargs)

    def parse(self):
        subcommand = self.args[0]
        id = self.args[1]

        if subcommand in ['volumes', 'thickness']:
            experiments = bx.xnat.collect_experiments(self.xnat, id,
                                                      max_rows=10)
            func = getattr(bx.braak, subcommand)
            df = func(self.xnat, experiments, resource_name='FREESURFER6_HIRES')
            self.to_excel(df)
        elif subcommand == 'fdg':
            df = self.run_id(id, dl.measurements, subfunc=subcommand,
                             resource_name='FDG_QUANTIFICATION', max_rows=10)
            self.to_excel(df)


def volumes(x, experiments, resource_name='FREESURFER6_HIRES'):
    debug = False
    braak_regions = {'Braak_I_II': (braak_I_II, braak_I_II_ctx),
                     'Braak_III_IV': (braak_III_IV, braak_III_IV_ctx),
                     'Braak_V_VI': (braak_V_VI, braak_V_VI_ctx)}

    columns = ['ID', 'region', 'measurement', 'value', 'subject']
    table = []
    for e in tqdm(experiments):
        try:
            res = x.select.experiment(e['ID']).resource(resource_name)
            if not res.exists():
                msg = 'Skipping %s. %s not found.' % (e['ID'], resource_name)
                log.error(msg)
                continue
            aparc = res.aparc()
            sl = e['subject_label']
            aseg = res.aseg()
            for n, (regions, regions_ctx) in braak_regions.items():
                total = []
                for r in regions_ctx:
                    for s in ['left', 'right']:
                        query = 'region == "{region}" & side == "{side}" & \
                                               measurement == "{measurement}"'
                        q = query.format(region=r, side=s, measurement='GrayVol')
                        value = float(aparc.query(q)['value'])
                        total.append(value)
                for r in regions:
                    query = 'region == "{region}" & measurement == "{measurement}"'
                    q = query.format(region=r, measurement='Volume_mm3')
                    value = float(aseg.query(q)['value'])
                    total.append(value)
                row = [e['ID'], n, 'Volume_mm3', sum(total), sl]
                table.append(row)
        except KeyboardInterrupt:
            return pd.DataFrame(table, columns=columns).set_index('ID')
        except Exception as exc:
            if debug:
                raise exc
            else:
                log.error('Failed for %s. Skipping it. (%s)' % (e, exc))

    return pd.DataFrame(table, columns=columns).set_index('ID')


def thickness(x, experiments, resource_name='FREESURFER6_HIRES'):
    debug = False
    braak_regions = [braak_I_II_ctx, braak_III_IV_ctx, braak_V_VI_ctx]

    names = ['Braak_I_II', 'Braak_III_IV', 'Braak_V_VI']
    columns = ['ID', 'Braak region', 'measurement', 'value', 'subject']
    table = []
    for e in tqdm(experiments):
        try:
            res = x.select.experiment(e['ID']).resource(resource_name)
            sl = e['subject_label']
            aparc = res.aparc()
            query = 'region == "{region}" & side == "{side}" & \
                         measurement == "{measurement}"'
            for r, n in zip(braak_regions, names):
                total_surf_area = 0
                weighted_sum = 0
                for b in r:
                    for s in ['left', 'right']:
                        q = query.format(region=b, side=s, measurement='ThickAvg')
                        thickness = float(aparc.query(q)['value'])

                        q = query.format(region=b, side=s, measurement='SurfArea')
                        surf_area = int(aparc.query(q)['value'])

                        weighted_sum += thickness * surf_area
                        total_surf_area += surf_area
                final = weighted_sum / total_surf_area
                row = [e['ID'], n, 'ThickAvg', final, sl]
                table.append(row)
        except KeyboardInterrupt:
            return pd.DataFrame(table, columns=columns).set_index('ID')
        except Exception as exc:
            if debug:
                raise exc
            else:
                log.error('Failed for %s. Skipping it. (%s)' % (e, exc))

    return pd.DataFrame(table, columns=columns).set_index('ID')



from bx.command import Command
from bx import download as dl


class DONSURFCommand(Command):
    """DONSURF - Diffusion ON SURFace

    Available subcommands:
     files:\t\tdownload all `recon-all` outputs (segmentation maps, files, everything...)
     aparc:\t\tcreate an Excel table with all `aparc` measurements
     snapshot:\t\tdownload a snapshot from the `recon-all` pipeline
     report:\t\tdownload the validation report issued by bbrc-validator
     tests:\t\tcreate an Excel table with all automatic tests outcomes from bbrc-validator

    Usage:
     bx donsurf <subcommand> <resource_id>

    References:
      - Montal V. et al., Alzheimers Dement, 2017. DOI: 10.1016/j.jalz.2017.09.013
    """
    nargs = 2
    resource_name = 'DONSURF'
    subcommands = ['aparc', 'snapshot', 'tests', 'report', 'files']
    validator = 'DONSURFValidator'
    url = 'https://gitlab.com/bbrc/xnat/xnat-pipelines/-/tree/master/donsurf'

    def __init__(self, *args, **kwargs):
        super(DONSURFCommand, self).__init__(*args, **kwargs)

    def parse(self):
        subcommand = self.args[0]
        id = self.args[1]  # should be a project or an experiment_id

        if subcommand in ['aparc']:
            df = self.run_id(id, dl.measurements,
                             resource_name=self.resource_name,
                             subfunc=subcommand, max_rows=10)
            df = df.query('measurement == "ThickAvg"')
            self.to_excel(df)

        elif subcommand in ['files', 'report', 'snapshot']:
            self.run_id(id, dl.download, resource_name=self.resource_name,
                        validator=self.validator, destdir=self.destdir,
                        subcommand=subcommand)

        elif subcommand == 'tests':

            version = ['*']
            from bx import validation as val
            df = self.run_id(id, val.validation_scores,
                             validator=self.validator,
                             version=version, max_rows=25)
            self.to_excel(df)



from bx.command import Command
from bx import download as dl


class FreeSurfer7Command(Command):
    """FreeSurfer v7.1.1

    Available subcommands:
     files:\t\tdownload all `recon-all` outputs (segmentation maps, files, everything...)
     aseg:\t\tcreate an Excel table with all `aseg` measurements
     aparc:\t\tcreate an Excel table with all `aparc` measurements
     hippoSfVolumes:\tsave an Excel table with hippocampal subfield volumes
     snapshot:\t\tdownload a snapshot from the `recon-all` pipeline
     report:\t\tdownload the validation report issued by bbrc-validator
     tests:\t\tcreate an Excel table with all automatic tests outcomes from bbrc-validator

    Usage:
     bx freesurfer7 <subcommand> <resource_id>
    """
    nargs = 2
    resource_name = 'FREESURFER7'
    subcommands = ['aparc', 'aseg', 'hippoSfVolumes', 'snapshot', 'tests',
                   'report', 'files']
    validator = 'FreeSurfer7Validator'
    url = 'https://gitlab.com/bbrc/xnat/docker-images/-/tree/master/freesurfer7'

    def __init__(self, *args, **kwargs):
        if self.resource_name == 'FREESURFER7':
            fs7_subcmds = ['amygNucVolumes', 'brainstem', 'thalamus',
                           'hypothalamus', 'jack']
            for cmd in fs7_subcmds:
                self.subcommands.append(cmd)

            s = 'amygNucVolumes:\tsave an Excel table with amygdalar volumes\n' \
                '     brainstem:\t\tsave an Excel table with brainstem substructures volumes\n'\
                '     thalamus:\t\tsave an Excel table with thalamic nuclei volumes\n'\
                '     hypothalamus:\tsave an Excel table with hypothalamic subunits volumes\n'\
                '     jack:\t\tcompute the cortical AD signature with FS7 results\n'\
                '     hippoSf'
            self.__doc__ = self.__doc__.replace('hippoSf', s)
        super(FreeSurfer7Command, self).__init__(*args, **kwargs)

    def parse(self):
        subcommand = self.args[0]
        id = self.args[1]  # should be a project or an experiment_id

        if subcommand in ['brainstem', 'thalamus', 'hypothalamus']:
            self.resource_name = 'FREESURFER7_EXTRAS'  # switch resource

        if subcommand in ['aparc', 'aseg', 'hippoSfVolumes', 'amygNucVolumes',
                          'brainstem', 'thalamus', 'hypothalamus']:
            import logging as log
            msg = 'FreeSurfer6 has a bug in the computation of some of the'\
                  ' global metrics when analyzing high resolution data'\
                  '(reported values are in voxels not mm3).'\
                  'For consistency those have been excluded from bx results. '\
                  'Source: https://surfer.nmr.mgh.harvard.edu/fswiki/BrainVolStatsFixed'
            if 'HIRES' in self.resource_name:
                log.warning(msg)
            df = self.run_id(id, dl.measurements,
                             resource_name=self.resource_name,
                             subfunc=subcommand, max_rows=10)
            self.to_excel(df)

        elif subcommand in ['jack']:
            from bx import signature as sig
            from bx import xnat
            experiments = xnat.collect_experiments(self.xnat, id, max_rows=10)

            df = sig.signatures(self.xnat, experiments, subcommand,
                                resource_name='FREESURFER7')
            self.to_excel(df)

        elif subcommand in ['files', 'report', 'snapshot']:
            self.subcommand_download()

        elif subcommand == 'tests':

            version = ['##0390c55f', '*']
            from bx import validation as val
            df = self.run_id(id, val.validation_scores,
                             validator=self.validator,
                             version=version, max_rows=25)
            self.to_excel(df)

    def subcommand_download(self):
        subcommand = self.args[0]
        id = self.args[1]

        resource_name = self.resource_name
        validator = self.validator
        if 'hires' in self.command:
            resource_name = 'FREESURFER6_HIRES'
            validator = 'FreeSurferHiresValidator'
        if 'extras' in self.command:
            resource_name = 'FREESURFER7_EXTRAS'
            validator = 'FreeSurfer7ExtrasValidator'

        from bx import download as dl
        self.run_id(id, dl.download, resource_name=resource_name,
                    validator=validator, destdir=self.destdir,
                    subcommand=subcommand)


class FreeSurfer6Command(FreeSurfer7Command):
    nargs = 2
    resource_name = 'FREESURFER6'
    validator = 'FreeSurferValidator'
    url = 'https://gitlab.com/bbrc/xnat/xnat-pipelines/-/tree/master/freesurfer'
    __doc__ = FreeSurfer7Command.__doc__.replace('7.1.1', '6.0.0')
    __doc__ = __doc__.replace('freesurfer7', 'freesurfer6')

    def __init__(self, *args, **kwargs):
        super(FreeSurfer6Command, self).__init__(*args, **kwargs)


class FreeSurfer6HiresCommand(FreeSurfer7Command):
    nargs = 2
    resource_name = 'FREESURFER6_HIRES'
    validator = 'FreeSurferHiresValidator'
    url = 'https://gitlab.com/bbrc/xnat/xnat-pipelines/-/tree/master/freesurfer'
    __doc__ = FreeSurfer7Command.__doc__.replace('7.1.1',
                                                 '6.0.0 (-hires option)')
    __doc__ = __doc__.replace('freesurfer7', 'freesurfer6hires')

    def __init__(self, *args, **kwargs):
        super(FreeSurfer6HiresCommand, self).__init__(*args, **kwargs)


class FreeSurfer7ExtrasCommand(FreeSurfer7Command):
    """FreeSurfer v7.2.0 (extra segmentation modules)

    Available subcommands:
     files:\t\tdownload all extra segmentation modules outputs (segmentation maps, files, everything...)
     snapshot:\t\tdownload a snapshot from the extra segmentation modules pipeline
     report:\t\tdownload the validation report issued by bbrc-validator
     tests:\t\tcreate an Excel table with all automatic tests outcomes from bbrc-validator

    Usage:
     bx freesurfer7extras <subcommand> <resource_id>
    """
    nargs = 2
    resource_name = 'FREESURFER7_EXTRAS'
    validator = 'FreeSurfer7ExtrasValidator'
    subcommands = ['snapshot', 'tests', 'report', 'files']
    url = 'https://gitlab.com/bbrc/xnat/xnat-pipelines/-/tree/master/freesurfer_extras'

    def __init__(self, *args, **kwargs):
        super(FreeSurfer7ExtrasCommand, self).__init__(*args, **kwargs)



from bx.command import Command
import logging as log
from nibabel.freesurfer import io
import os.path as op
import bx


class SignatureCommand(Command):
    """Download composite measurements labeled as 'signatures' of Alzheimer's Disease

    Available subcommands:
     jack:\t\tbased on FreeSurfer's cortical thickness and local cortical gray matter volumes
     dickerson:\t\tbased on Dickerson's cortical signatures (see references below)

    Usage:
     bx signature <subcommand> <resource_id>

    Jack's AD signature is calculated in two versions,
    weighted and not weighted. Weighted means that the formula has been
    applied by normalizing each ROI value by local surface area (as explained
    in the papers).
    Not-weighted versions correspond to mean values across regions.

    Examples:

    `bx signature jack` will return Jack's signature, based on thickness and grayvol values.
    `bx signature dickerson` will return AD and aging signatures, based only on
    thickness values as they do not have any "grayvol" version.

    References:
    - Jack et al., Alzheimers Dement. 2017
    - Dickerson et al., Neurology, 2011
    - Bakkour et al., NeuroImage, 2013
    """
    nargs = 2
    subcommands = ['jack', 'dickerson']
    url = 'https://gitlab.com/bbrc/xnat/docker-images/-/tree/master/freesurfer7'

    def __init__(self, *args, **kwargs):
        super(SignatureCommand, self).__init__(*args, **kwargs)

    def parse(self):
        subcommand = self.args[0]
        id = self.args[1]  # should be a project or an experiment_id

        from bx import xnat
        experiments = xnat.collect_experiments(self.xnat, id, max_rows=10)

        if subcommand in ['jack']:
            df = signatures(self.xnat, experiments, subcommand,
                            resource_name='FREESURFER7')
            self.to_excel(df)
        elif subcommand in ['dickerson']:
            df = signatures(self.xnat, experiments, subcommand,
                            resource_name='FREESURFER7')
            q = 'signature == "ad" & weighted or signature == "aging" & weighted'
            self.to_excel(df.query(q))


def __jack_signature__(x, experiment_id, regions, weighted=True,
                       measurement='ThickAvg', resource_name='FREESURFER7'):
    e = x.select.experiment(experiment_id)
    r = e.resource(resource_name)
    aparc = r.aparc()

    weighted_sum = 0
    total_surf_area = 0

    query = 'region == "{region}" & side == "{side}" & \
             measurement == "{measurement}"'

    for r in regions:
        for s in ['left', 'right']:
            q = query.format(region=r, side=s, measurement=measurement)
            thickness = float(aparc.query(q)['value'])

            q = query.format(region=r, side=s, measurement='SurfArea')
            surf_area = int(aparc.query(q)['value'])

            weighted_sum += thickness * surf_area if weighted else thickness
            total_surf_area += surf_area

    if weighted:
        final = weighted_sum / total_surf_area
    else:
        final = weighted_sum / (2 * len(regions))

    return final


def __coregister__(r, rois_path):
    """ Download cortical surface models, coregister them with fsaverage and
    resample the labels with respect to the coregistered surfaces.

    Params:
    - r             pyxnat FREESUFER-type resource
    - rois_path     Path to the .label files (eg. those in bx/data/dickerson) """

    import os
    import tempfile
    from glob import glob

    # Cortical lh white and sphere
    wd = tempfile.mkdtemp()
    print(wd)
    subject = 'subject'
    os.mkdir(op.join(wd, 'subject'))
    wd = op.join(wd, 'subject')
    os.mkdir(op.join(wd, 'surf'))

    for side in ['lh', 'rh']:
        for item in ['sphere.reg', 'white']:
            each = '%s.%s' % (side, item)
            f = list(r.files('*%s' % each))[0]
            f.get(op.join(wd, 'surf', each))

    os.environ['SUBJECTS_DIR'] = op.dirname(wd)
    fsaverage = op.join(os.environ['FREESURFER'], 'subjects', 'fsaverage')
    cmd = 'ln -s %s %s' % (fsaverage, op.dirname(wd))
    os.system(cmd)

    files = glob(op.join(rois_path, '*.label'))
    cmd = 'mri_label2label --srclabel {label} --srcsubject fsaverage '\
          '--trglabel {outfile} --trgsubject {subject} --regmethod surface '\
          '--hemi {hemi}'

    for label in files:
        hemi = label.split('/')[-1].split('.')[0]
        fp = op.join(wd, label)
        c = cmd.format(label=label, outfile=fp, subject=subject, hemi=hemi)
        print(c)
        os.system(c)
    return wd


def __signature__(x, experiment_id, rois_path, weighted=True,
                  resource_name='FREESURFER7'):
    """ Calculating signature based on FreeSurfer .label files """
    import tempfile
    import os
    from glob import glob

    fh, fp = tempfile.mkstemp(suffix='.thickness')
    os.close(fh)
    e = x.select.experiment(experiment_id)
    r = e.resource(resource_name)

    # Cortical thickness map
    f = list(r.files('*lh.thickness'))[0]
    f.get(fp)
    lh_thickness = io.read_morph_data(fp)
    os.remove(fp)

    f = list(r.files('*rh.thickness'))[0]
    f.get(fp)
    rh_thickness = io.read_morph_data(fp)
    os.remove(fp)

    files = glob(op.join(rois_path, '*.label'))
    thickness_sum = 0
    n_regions = 0  # len(files)
    n_vertex = 0

    for f in files:
        if 'adsig' in f: continue
        thickness = 0
        n_regions += 1
        filename = op.basename(f)
        roi = io.read_label(f)
        n_vertex += len(roi)
        if filename.startswith('rh'):
            for vertex in roi:
                thickness += rh_thickness[vertex]
        elif filename.startswith('lh'):
            for vertex in roi:
                thickness += lh_thickness[vertex]

        if weighted:
            thickness_sum += thickness
        else:
            thickness_sum += thickness / len(roi)

    if weighted:
        signature = thickness_sum / n_vertex
    else:
        signature = thickness_sum / n_regions

    return signature


def signature(x, experiment_id, subcommand, resource_name='FREESURFER7'):
    import pandas as pd

    columns = ['signature', 'weighted', 'measurement', 'value']
    table = []
    # Jack's signature
    if subcommand == 'jack':
        for weighted in [False, True]:
            for m in ['ThickAvg', 'GrayVol']:
                regions = ['entorhinal', 'inferiortemporal', 'middletemporal',
                           'fusiform']
                res = __jack_signature__(x, experiment_id, regions, weighted,
                                         m, resource_name=resource_name)

                row = ['jack', weighted, m, res]
                table.append(row)
    # ad and Aging signatures
    elif subcommand == 'dickerson':
        import tempfile
        e = x.select.experiment(experiment_id)
        r = e.resource('DICKERSON')
        wd = tempfile.mkdtemp()
        r.get(wd, extract=True)
        wd = op.join(wd, 'DICKERSON')

        for sig in ['ad', 'aging']:
            for m in ['ThickAvg']:
                for weighted in [False, True]:
                    res = __signature__(x, experiment_id, op.join(wd, sig), weighted,
                                        resource_name=resource_name)
                    row = [sig, weighted, m, res]
                    table.append(row)

    return pd.DataFrame(table, columns=columns)


def signatures(x, experiments, subcommand, resource_name='FREESURFER7'):
    from tqdm import tqdm
    import pandas as pd

    table = []
    for e in tqdm(experiments):
        log.debug(e)
        try:
            s = e['subject_label']
            e_id = e['ID']
            volumes = signature(x, e_id, subcommand,
                                resource_name=resource_name)

            volumes['subject'] = s
            volumes['ID'] = e['ID']
            table.append(volumes)
        except KeyboardInterrupt:
            return pd.concat(table).set_index('ID').sort_index()
        except Exception as exc:
            log.error('Failed for %s. Skipping it. (%s)' % (e, exc))

    data = pd.concat(table).set_index('ID').sort_index()
    return data



import setuptools
from setuptools import setup
from bx import __version__


import os.path as op
this_directory = op.abspath(op.dirname(__file__))
with open(op.join(this_directory, 'README.md')) as f:
    long_description = f.read()

download_url = 'https://gitlab.com/bbrc/xnat/bx/-/archive/v%s/bx-v%s.tar.gz'
download_url = download_url % (__version__, __version__)

setup(
    name='bbrc-bx',
    install_requires=['coverage>=4.5',
                      'pytest>=7.1',
                      'pytest-cov>=4.0',
                      'requests>=2.21',
                      'six>=1.10',
                      'lxml>=4.3',
                      'nibabel>=2.3',
                      'pydicom>=1.2',
                      'numpy>=1.16',
                      'pandas>=0.24',
                      'xlrd>=1.2',
                      'xlutils>=2.0',
                      'xlwt>=1.3',
                      'tqdm>=4.31',
                      'openpyxl>=2.6',
                      'bbrc-pyxnat>=1.4.2',
                      'rich>=10.1'],
    scripts=['bin/bx', 'bin/dump'],
    long_description=long_description,
    long_description_content_type='text/markdown',
    version=__version__,
    description='BarcelonaBeta + XNAT = bx',
    packages=setuptools.find_packages(),
    author='Greg Operto, Jordi Huguet',
    author_email='goperto@barcelonabeta.org',
    url='https://gitlab.com/bbrc/xnat/bx',
    download_url=download_url,
    classifiers=['Intended Audience :: Science/Research',
                 'Intended Audience :: Developers',
                 'Topic :: Scientific/Engineering',
                 'Operating System :: Unix',
                 'Programming Language :: Python :: 3.7',
                 'Programming Language :: Python :: 3.8'],
    include_package_data=True

)



from bx.command import Command
from bx import download as dl


class BAMOSCommand(Command):
    """BAMOS (Bayesian MOdel Selection for white matter lesion segmentation)

    Available subcommands:
     files:\t\tdownload all `BAMOS` outputs
     volumes:\t\tcreate an Excel table with global lesion volumes
     layers:\t\tdownload `layer` (i.e. depth) maps
     lobes:\t\tdownload lobar segmentation maps
     stats:\t\tcreate an Excel table with lesions stats per lobe and depth
     snapshot:\t\tdownload a snapshot from the `BAMOS` pipeline
     report:\t\tdownload the validation report issued by `BAMOSValidator`
     tests:\t\tcreate an Excel table with all automatic tests outcomes from bbrc-validator


    Usage:
     bx bamos <subcommand> <resource_id>

    References:
    - Sudre et al., IEEE TMI, 2015
    """
    nargs = 2
    resource_name = 'BAMOS'
    subcommands = ['volumes', 'files', 'layers', 'lobes', 'stats', 'snapshot',
                   'report', 'tests']
    validator = 'BAMOSValidator'
    url = 'https://gitlab.com/bbrc/xnat/docker-images/-/tree/master/bamos'


    def __init__(self, *args, **kwargs):
        super(BAMOSCommand, self).__init__(*args, **kwargs)

    def parse(self, test=False):
        subcommand = self.args[0]
        id = self.args[1]

        if subcommand in ['files', 'report', 'snapshot', 'layers', 'lobes']:
            self.run_id(id, dl.download, resource_name=self.resource_name,
                        validator=self.validator, destdir=self.destdir,
                        subcommand=subcommand)

        elif subcommand in ['volumes', 'stats']:
            sf = 'bamos_%s' % subcommand
            df = self.run_id(id, dl.measurements, subfunc=sf,
                             resource_name=self.resource_name, max_rows=10)
            self.to_excel(df)
        elif subcommand == 'tests':
            version = ['*', '4e37c9d0']
            from bx import validation as val
            df = self.run_id(id, val.validation_scores,
                             validator=self.validator,
                             version=version, max_rows=25)
            self.to_excel(df)



from bx.command import Command
from bx import download as dl


class ANTSCommand(Command):
    """ANTS - Advanced Normalization Tools

    Available subcommands:
     files:\t\tdownload all `ANTS` outputs
     snapshot:\t\tdownload a snapshot from the `ANTS` pipeline
     report:\t\tdownload the validation report issued by `ANTSValidator`
     tests:\t\tcreates an Excel table with all automatic tests outcomes from `ANTSValidator`

    Usage:
     bx ants <subcommand> <resource_id>
    """
    nargs = 2
    resource_name = 'ANTS'
    subcommands = ['files', 'report', 'snapshot', 'tests']
    validator = 'ANTSValidator'
    url = 'https://gitlab.com/bbrc/xnat/xnat-pipelines/-/tree/master/ants'


    def __init__(self, *args, **kwargs):
        super(ANTSCommand, self).__init__(*args, **kwargs)

    def parse(self):
        subcommand = self.args[0]
        id = self.args[1]  # should be a project or an experiment_id

        if subcommand in ['files', 'report', 'snapshot']:
            self.run_id(id, dl.download, resource_name=self.resource_name,
                        validator=self.validator, destdir=self.destdir,
                        subcommand=subcommand)

        elif subcommand == 'tests':
            version = ['*', '0390c55f']

            from bx import validation as val
            df = self.run_id(id, val.validation_scores,
                             validator=self.validator,
                             version=version, max_rows=25)
            self.to_excel(df)
